;-------------------------------------------------------------------------------
; NAME: I2C.asm
;
; LABEL PREFIX: "XCVR"
;
; Author:	G8KBB
;
; PURPOSE:	This is the I2C interface functions ( rest is in ISR ).
;
;
;REVISION HISTORY:
;		15/06/99	G8KBB	Initial Revision
;		10/5/00		G8KBB	Port to 17C766
;
;-------------------------------------------------------------------------------		
;
; I2C Buffer Usage
;
; Byte		Function
;  0		State
;  1		Result
;  2		I2C Slave Address
;  3		I2C Device Sub-address
;  4		Byte count for transfer
;  5		RAM Page ( BSR image )
;  6		RAM Start address pointer
;  7		Command flags
;  8 - 0x0b	spare space for small data messages
;		for larger messages, get a buffer and mark as '-3'
;
; State  takes the following values
; -3 ( 0xfd ) Buffer in use as data buffer
; -2 ( 0xfe ) Retry active - waiting
; -1 ( 0xff ) Command complete, buffer not in use by I2C software
; 0 Idle - buffer free and available for use
; 1 Pending - command present awaiting processing by handler
; 2..N Internal operating states
;
; Result bit mask:
; 0 if set, busy / in progress, clear = idle
; 1 if set, OK, if clear an error occurred
; 2 if set, slave is active, if clear slave inactive
; 3 if set, an ACK error occurred
; 4 if set, a fatal error occurred
; 5 if set, a timeout happened
; 6 if set, transfer was aborted
; 7 if set, data transfer succeeded
;
; Command flags
; 0 set = read, clear = write
; 1 set = sub address present and must be sent
; 2 if set, retry if slave is inactive
; 3 if set, automatically clear buffer on completion
; 4 \
; 5  \ retry counter if slave inactive ( 10's of milliseconds )
; 6  / note first delay will be 0-10 msec depending on timing
; 7 /
;
; ------------------------------------------------------------------------------
;
; I2C_Startup
;	used to kick off I2C handler if passive
;	Assumes that:
;		FSR1 points to a new command buffer
;		Ram bank in FSR already set up to point to I2C ram buffer page
;	CHANGES:
;		bsr - will point to register bank 6 on exit if startup happens
;
I2C_Startup:
	bsf	CPUSTA,GLINTD		; disable all interrupts
	btfsc	I2C_STATE, I2C_Active	; if already active
	goto	I2C_Startup_pending	; then mark new as pending
	movpf	FSR1, I2C_PTR		; else initialise pointer to new command
	movlw	2			; step FSR1 on to command data
	addwf	FSR1,F
	bsf	ALUSTA,FS2		; set FSR1 to auto increment
	bcf	ALUSTA,FS3
	movpf	INDF1,I2C_address	; now copy command data to current
	movpf	INDF1,I2C_subaddr
	movpf	INDF1,I2C_count
	movpf	INDF1,I2C_page
	movpf	INDF1,I2C_ram_ptr
	movpf	INDF1,I2C_command
	bsf	ALUSTA,FS3		; reinstate FSR1 no auto change
	movfp	I2C_PTR,FSR1		; point to start of command buffer again
	movlw	2			; and force to state 2 - state 1 code follows
	movwf	INDF1
	incf	FSR1,F			; point to result
	clrf	INDF1,F			; clear flags
	movlb	BANK6			; point to I2C registers
	movfp	SSPCON2,WREG		; check no command in progress
	andlw	I2C_FUNC_BITS
	btfss	ALUSTA,Z		
	goto	I2C_Startup_error	; if so, error
	btfsc	SSPSTAT, I2C_Idle	; check idle
	goto	I2C_Startup_error	; if not, error
	bsf	SSPCON2,SEN		; initiate start command on I2C bus
	bsf	INDF1,0			; mark command as active
	bsf	I2C_STATE, I2C_Active	; mark handler as active
	movfp	I2C_PTR,FSR1		; point to start of command buffer again
I2C_Startup_exit:
	bcf	CPUSTA,GLINTD		; enable all interrupts
	return
I2C_Startup_error:
	movfp	I2C_PTR,FSR1		; point to start of command buffer again
	clrf	I2C_PTR,F			; disable current command pointer
I2C_Startup_pending:
	movlw	1			; make new command state 1 ( pending )
	movwf	INDF1
	incf	FSR1,F			; remove 'active' flag
	bcf	INDF1,0
	decf	FSR1,F
	goto	I2C_Startup_exit
;
; --------------------------------------------------------------------------
;
; I2C_10msec
;	called by 10 msec command loop, check for commands where
;	slave was not active and retry was called for.
;	attempts a retry for all such found
;
I2C_10msec:
	movlr	BANK2		 	; point to page with I2C data
	movlw	LOW I2C_FIRST_BUF	; polint to first buffer
	movwf	FSR1
I2C_10msec_loop:
	movlw	-2			; find every entry set to -2
	cpfseq	INDF1			; because these need a restart
	goto	I2C_10msec_skip		; skip over if not set to -2
	movlw	7			; point to command byte
	addwf	FSR1,F
	swapf	INDF1,W			; get count nibble in WREG[0..3]
	andlw	0x0f
	btfsc	ALUSTA,Z		; if zero,
	bcf	INDF1,2			; clear retry flag in command
	btfss	ALUSTA,Z		; otherwise..
	decf	WREG,F			; .. decrement 10msec retry counter
	movwf	I2C_Temp		; save new counter ( if changed )
	movlw	0x0f			; mask old count in command to 0000
	andwf	INDF1,F
	swapf	I2C_Temp,W		; put new count in top nibble of WREG
	iorwf	INDF1,F			; and replace in command byte
	movlw	-7			; point back to status byte
	addwf	FSR1,F
;	movlw	1			; set to 'pending'..
;	movwf	INDF1			; ..happens automatically in I2C_Startup
	call	I2C_Startup		; and have another go
I2C_10msec_skip:
	movlw	I2C_BUF_SIZE		; step on to next structure
	addwf	FSR1,F
	movlw	LOW (I2C_BUF_END-1)	; are we at the end ?
	cpfsgt	FSR1
	goto	I2C_10msec_loop		; if not, loop back thru all
	clrf	BSR,F			; else reinstate BSR and return
	return
;
; --------------------------------------------------------------------------
;
; I2C_Find_Buf
;
; locates a free buffer.
; returns with Z clear if one found and FSR1 pointing to it
; returns with Z set and FSR1 holding zero if none are free
; BSR set to I2C ram page if succeeds, unchanged if not
;

I2C_Find_Buf:
	movpf	BSR,WREG
	movlr	BANK2			; point to page with buffers in
	movwf	I2C_Temp		; save BSR in case of error
	movlw	LOW I2C_FIRST_BUF	; start at the first
	movwf	FSR1
I2C_Find_Buf_Loop:
	tstfsz	INDF1			; find one with first byte 00
	goto	I2C_Find_Buf_Skip
	bcf	ALUSTA,Z		; clear zero flag to slow buffer found
	return				; and return
I2C_Find_Buf_Skip:
	movlw	I2C_BUF_SIZE		; else step on to next one
	addwf	FSR1,F
	movlw	LOW (I2C_BUF_END-1)	; are we at the end ?
	cpfsgt	FSR1
	goto	I2C_Find_Buf_Loop	; if not, keep looking
	movfp	I2C_Temp,BSR		; error - reinstate BSR
	clrf	FSR1,F			; and return an error
	bsf	ALUSTA,Z
	return

;
; --------------------------------------------------------------------------
;
; I2C_Find_Buf2
;
; locates a free buffer.
; returns with Z clear if one found and:
;	FSR1 pointing to it
;	FSR0 pointing to I2C Slave Address
; returns with Z set and FSR1 holding zero if none are free
; BSR set to I2C ram page if succeeds, unchanged if not
;
I2C_Find_Buf2:
	call	I2C_Find_Buf
	btfsc	ALUSTA,Z
	return
	movpf	FSR1, FSR0
	incf	FSR0,F
	incf	FSR0,F
	bcf	ALUSTA,Z
	return

