;-------------------------------------------------------------------------------
; NAME: Isr.asm
;
; LABEL PREFIX: "XCVR"
;
; Author:	G8KBB
;
; PURPOSE:	This routine services interrupts.
;
;
;REVISION HISTORY:
;		15/06/99	G8KBB	Initial Revision
;		10/5/00		G8KBB	Port to 17C766
;
;-------------------------------------------------------------------------------		
;-------------------------------------------------------------------------------		
;
IsrT0IF:
;
; 1ms clock tick
; timer 0 overflow, every 1msec. Increment counter & exit
; with 32 MHz clock, timer0 clock is Fosc/4
; so counter needs to overflow after 8000 cycles
; no prescaler
; note we offset by 10 for cpu cycles lost by isr.
; program saves registers, then reads timer 0 overflow count
; into UNBANK5:6. It then adds timer setting to it so count
; carries forward the overflow from previous timer count
;
; also performs contact debounce for PTT line - see comments in next function
;
; subject to keyboard type, also performs keyboard scan
;
; note that the following code is performed in start.asm
; to ensure pclath is not corrupted
;
;	MOVFP 	ALUSTA, UNBANK1 ; Push ALUSTA value
;	MOVFP 	BSR, 	UNBANK2 ; Push BSR value
;	MOVFP 	WREG, 	UNBANK3 ; Push WREG value
;	MOVFP 	PCLATH, UNBANK4 ; Push PCLATH value

	clrf	BSR,F

	MOVPF TMR0L, UNBANK6 	;read low tmr0
	MOVPF TMR0H, UNBANK5 	;read high tmr0
	MOVFP UNBANK6, WREG 	;tmplo -> wreg
	CPFSLT TMR0L 		;tmr0l < wreg?
	goto	isrt0if0	;no then read it OK
	MOVPF TMR0L, UNBANK6 	;yes, read again ! read low tmr0
	MOVPF TMR0H, UNBANK5 	;read high tmr0

; now 'subtract' T0INIT to give another 1msec delay
; keeping overrun from previous period
isrt0if0:
	movlw	(0-T0INIT) & 0xff
	addwf	UNBANK6,F
	movlw	((0-T0INIT) >> 8) & 0xff
	addwfc	UNBANK5,F

	MOVFP 	UNBANK6, TMR0L 	; reinitialise timer counter
	MOVFP 	UNBANK5, TMR0H 	;
	incf	ONE_MSEC,F	; bump 1 msec counter

	movlr	BANK1		; ptt debounce timer in ram page 1
	tstfsz	ptt_counter	; if not zero skip over and ignore
	decfsz	ptt_counter,F	; else wait ptt_counter milliseconds
	goto	isrt0not_ptt	; otherwise ( zero ) ignore
	;	movlr	BANK0
	;	movlb	BANK0
	clrf	BSR,F			; do same as above 2 mov instructions
;	bsf	UPDATE_2,XMIT_CHANGED	; flag xmit state change
	bcf	RXFlags2,RXF2_PTT_active ; new state receive
	btfss	PORTA,PTT		; unless PTT line low
	bsf	RXFlags2,RXF2_PTT_active ; in which case state is transmit
isrt0not_ptt:
#ifdef	KBD_4X4
	clrf	BSR,F
	btfss	ONE_MSEC,0		; only do this every 2 msec
	goto	isrt0end		; so skip every other msec
	movlr	BANK1			; keyboard scan data in page 1
	movpf	FSR1,UNBANK5		; save FSR1 as we will use it
	movlb	BANK7+1			; port H/J are in bank 8
	movfp	SW_scancount,WREG	; get current index [0..3]
	andlw	3			; defensive - get 2 bits only
	addlw	LOW SW_scanbase		; index into scan data
	movwf	FSR1			; with FSR/INDF1
#ifdef SWAP_SWITCH_PORTS
	movpf	PORTJ,INDF1		; read port & store in array
#else
	movpf	PORTH,INDF1		; read port & store in array
#endif
	addlw	4			; step pointer on to second set
	movwf	FSR1			; with FSR/INDF1
#ifdef SWAP_SWITCH_PORTS
	movpf	PORTH,INDF1		; read port & store in array
#else
	movpf	PORTJ,INDF1		; read port & store in array
#endif
	incf	SW_scancount,F		; bump counter
	movlw	3			; mod 4 to give value of
	andwf	SW_scancount,F		; (n+1) mod 4 i.e. 0..3
	movfp	SW_scancount,WREG	; read new counter
isrlabtst:
	addwf	PCL,F			; index into jump table
	goto	isrt0s1			; go here for count = 0
	goto	isrt0s2			; go here for count = 1
	goto	isrt0s3			; go here for count = 2
	movlw	0xf7			; count = 3, bit 3 low
	goto	isrt0s
isrt0s3:
	movlw	0xfb			; count = 2, bit 2 low			
	goto	isrt0s
isrt0s2:
	movlw	0xfd			; count = 1, bit 1 low
	goto	isrt0s
isrt0s1:
	movlw	0xfe			; count = 0, bit 0 low
isrt0s:
	movwf	PORTH			; output new scan code
	movwf	PORTJ			; output new scan code
	movfp	UNBANK5,FSR1		; reinstate FSR1
#if (( HIGH isrt0s1 ) != ( HIGH IsrT0IF ))
#error jump table spans boundary of a page !
#endif

#endif

isrt0end:
	MOVFP 	UNBANK4, PCLATH ; Restore PCLATH value
	MOVFP 	UNBANK3, WREG 	; Restore WREG value
	MOVFP 	UNBANK2, BSR 	; Restore BSR value
	MOVFP 	UNBANK1, ALUSTA ; Restore ALUSTA value
;
	retfie

;-------------------------------------------------------------------------------		
;-------------------------------------------------------------------------------		
;
IsrPEIF:
;
; Peripheral Interrupt function
;
; interrupt handler for port B status change ( Opto interrupt )
; interrupt handler for I2C completion events
;
; note following code is done in start.asm
; to avoid corruption of pclath
;	MOVFP 	ALUSTA, UNBANK1 ; Push ALUSTA value
;	MOVFP 	BSR, UNBANK2 	; Push BSR value
;	MOVFP 	WREG, UNBANK3 	; Push WREG value
;	MOVFP 	PCLATH, UNBANK4 ; Push PCLATH value

;-------------------------------------------------------------------------------		
; Port B Interrupt Service Routine (ISR) code
;
	movlr	BANK0		; make sure ram page 0
	movlb	BANK1		; register bank 1 for PIR1
	btfss	PIR1,RBIF	; was interrupt due to port B ?
	goto	isr_not_portb	; if not, skip over
; port B ISR
	movlb	BANK0			; register bank 0 for portB
	movpf	PORTB,PORTB		; read port B to clear interrupt condition
	movpf	PORTB,UNBANK5		; and get port B new values ( INT_I )
	movlb	BANK1
	bcf	PIR1,RBIF		; clear port B interrupt flags
	movpf	PORTC,UNBANK6		; read channel B data from bus ( INT_K )
	movfp	UNBANK5,WREG		; now to find bits that changed...
	xorwf	OLD_PORT_B,W		; exor old and new values
;; replaced INT_J with WREG 20/02/01
;;	movwf	INT_J			; and store in J
;;	btfss	INT_J,OPTO_1_A		; check opto 1 interrupt
	btfss	WREG,OPTO_1_A		; check opto 1 interrupt
	goto	isr_o2_test		; and if no change go do opto 2
	btfss	UNBANK5,OPTO_1_A	; If channel A has gone low
	goto	isr_o1_anti		; then go treat as other sense
	btfss	UNBANK6,OPTO_1_B	; else normal- if channel B low
	goto	isr_o1_down		; then call it 'down'
isr_o1_up:				; else call it 'up'
	incf	OPTO_1_COUNT,F		; increment count
	goto	isr_o2_test		; end of opto 1 handler
isr_o1_anti:				; sense reversed
	btfss	UNBANK6,OPTO_1_B		; else opposite - if channel B low
	goto	isr_o1_up		; then call it up
isr_o1_down:				; else call it down
	decf	OPTO_1_COUNT,F
isr_o2_test:				; now do exactly same for channel B
;;	btfss	INT_J,OPTO_2_A		; replaced INT_J with WREG - see above 20/02/01
	btfss	WREG,OPTO_2_A
	goto	isr_o2_end
	btfss	UNBANK5,OPTO_2_A
	goto	isr_o2_anti
	btfss	UNBANK6,OPTO_2_B
	goto	isr_o2_down
isr_o2_up:
	incf	OPTO_2_COUNT,F
	goto	isr_o2_end
isr_o2_anti:
	btfss	UNBANK6,OPTO_2_B
	goto	isr_o2_up
isr_o2_down:
	decf	OPTO_2_COUNT,F
isr_o2_end:
	movpf	UNBANK5,OLD_PORT_B
isr_not_opto:
;
; if we had any other port B interrupt events other than opto
; then this is where they would go
;
isr_not_portb:
;
;-------------------------------------------------------------------------------		
; I2C INTERRUPT HANDLER
;
	movfp	FSR1, UNBANK5		; save FSR1
	movlb	BANK4			; access to PIR2
	btfss	PIR2, BCLIF		; bus collission ?
	goto	isr_check_i2c		; no - so must be normal event
	; bus collision has occurred
	bcf	PIR2, BCLIF		; reset interrupt flags
	bcf	PIR2, SSPIF
	movlb	BANK6			; access I2C register bank
	movlr	BANK2			; ram page with I2C data stores
	movlw	B'10010000'		; initialise ALUSTA so FSR1
	movwf	ALUSTA			; does not increment
isr_i2c_error:
	tstfsz	I2C_PTR			; make sure we have a valid pointer
	goto	isr_i2c_err_0		; if so continue
	goto	isr_i2c_pause		; if not, bomb out
isr_i2c_err_0:
	movfp	I2C_PTR,FSR1		; get current I2C buffer
	incf	FSR1,F			; point to result register
	bsf	INDF1,6			; set ABORT flag
	bsf	INDF1,4			; set FATAL ERROR flag
	bcf	INDF1,0			; clear BUSY flag
	decf	FSR1,F			; point back to state register
;
; I2C STATE 12 AND common exit code when a bus operation has ended
;
isr_i2c_state12:
isr_i2c_end:
	setf	INDF1,F			; set status to 0xFF ( finished )
	btfss	I2C_command,3		; should we automatically clear buffer ?
	goto	isr_i2c_nextone		; no - skip over following code
	clrf	INDF1,F			; set status to 00 ( buffer free )
	swapf	I2C_page,W		; get ram bank address in W[0..3]
	andlw	0x0f			; clear bits 4..7 so we have ram bank only
	sublw	2			; now see if it was ram bank 2
	btfss	ALUSTA,Z		; as if it was, assume it is one of our buffers
	goto	isr_i2c_nextone		; if not, skip over
	movfp	I2C_ram_ptr,FSR1	; now get ram pointer in page 2
	movfp	INDF1,WREG		; and read its status byte
	addlw	3			; and see if it is 3 ( buffer used as data buffer )
	btfsc	ALUSTA,Z		; if not, skip over
	clrf	INDF1,F			; else mark buffer as free ( 00 )
	movfp	I2C_PTR,FSR1		; point back to current I2C buffer
;
; look for next I2C command to execute by checking other buffers
;
isr_i2c_nextone:
	movlw	I2C_BUF_SIZE		; step pointer on
	addwf	FSR1,F			; to next I2C buffer
	movlw	LOW (I2C_BUF_END-1)	; and if at end of buffers
	cpfsgt	FSR1
	goto	isr_i2c_next_ok
	movlw	LOW I2C_FIRST_BUF	; then wrap pointer back to first
	movwf	FSR1			; and set FSR1 to new buffer
isr_i2c_next_ok:
	movfp	FSR1,WREG		; now see if we got back to where we started
	cpfseq	I2C_PTR			; because if so, no other buffers contain
	goto	isr_i2c_is_next_pending	; I2C requests to service
;
; in which case, pause I2C event handler
;
isr_i2c_pause:
	bcf	I2C_STATE, I2C_Active	; mark I2C handler as inactive
	clrf	I2C_PTR,F		; reset pointer
	goto	isr_i2c_exit		; and exit
;
; check if next buffer holds a command to process.
; If so, it will contain a value of 1 in its state register
; if not, nothing to do in this buffer, loop back checking
;
isr_i2c_is_next_pending:
	decfsz	INDF1,W			; check if '1'
	goto	isr_i2c_nextone		; if not loop
; new one found, start it up
	movpf	FSR1, I2C_PTR		; else initialise pointer to new command
	movlw	2			; step FSR1 on to command data
	addwf	FSR1,F
	bsf	ALUSTA,FS2		; set FSR1 to auto increment
	bcf	ALUSTA,FS3
	movpf	INDF1,I2C_address	; now copy command data to current
	movpf	INDF1,I2C_subaddr	; value working stores for ease
	movpf	INDF1,I2C_count		; of access
	movpf	INDF1,I2C_page
	movpf	INDF1,I2C_ram_ptr
	movpf	INDF1,I2C_command
	bsf	ALUSTA,FS3		; reinstate FSR1 no auto change
	movfp	I2C_PTR,FSR1		; point to start of command buffer again
;
; drop thru to i2c state 1 code
;
; STATE 1 
; We have a new I2C operation to process.
; If bus is idle ( should be ) then initiate an I2C START
; and set next state to 2 for completion of start command
;
isr_i2c_state1:
	incf	FSR1,F			; point to result
	clrf	INDF1,F			; clear I2C result flags
	movfp	SSPCON2,WREG		; check no command in progress
	andlw	I2C_FUNC_BITS
	btfss	ALUSTA,Z		
	goto	isr_i2c_internal_error	; if so, error
	btfsc	SSPSTAT, I2C_Idle	; check idle
	goto	isr_i2c_internal_error	; if not, error
	bsf	SSPCON2,SEN		; initiate start command on I2C bus
	movlw	2
;
; common exit code for I2C finite state machine
; Next state should be in WREG.
;
isr_i2c_state_common_exit:
	movfp	I2C_PTR,FSR1		; reinstate FSR1 pointing to I2C command
	movwf	INDF1			; and set next state from WREG
;
; common exit code that does NOT set state
;
isr_i2c_state_short_exit:
	btfss	SSPCON1,WCOL		; check no bus collision
	goto	isr_i2c_state_exit	; if not, skip out
	bcf	SSPCON1,WCOL		; else clear collision flag
	goto	isr_i2c_error		; and go to error handler
;
; catastrophic error handler.
;
isr_i2c_internal_error:
	movlw	B'00011000'		; reset I2C and stop MSSP hardware
	movwf	SSPCON1
	bsf	SSPCON1,SSPEN		; restart MSSP hardware
	goto	isr_i2c_error		; and go to error handler
;
;-------------------------------------------------------------------------------		
; check I2C interrupt
;
isr_check_i2c:
	btfss	PIR2, SSPIF		; was interrupt caused by I2C event ?
	goto	isr_not_i2c		; if not, skip out
	bcf	PIR2, SSPIF		; reset I2C interrupt event flag
	movlr	BANK2			; ram page with I2C data stores
	movlw	B'10010000'		; initialise ALUSTA so FSR1
	movwf	ALUSTA			; does not increment
	tstfsz	I2C_PTR			; have we a current buffer pointer ?
	goto	isr_i2c_exec		; if so, go process finite state machine
	goto	isr_not_i2c		; otherwise error - wasn't us ? !
;
isr_i2c_exec:
	movlb	BANK6			; point to I2C MSSP registers
	movfp	I2C_PTR,FSR1		; point to current command
	movlw	I2C_MAX_STATE - I2C_STATE_TABLE ; check that state is valid
	cpfslt	INDF1			; ie for which there is a table entry
	goto	isr_i2c_internal_error	; otherwise catastrophic error !
	movfp	INDF1,WREG		; fetch state
	addwf	PCL,F			; jump into jump table

;  WARNING - code from above line to end of state table must not span 256 word boundary !
#if	( HIGH I2C_MAX_STATE ) != ( HIGH isr_i2c_exec )
	error	ERROR - addwf PCL code must not span page boundary (I2C_STATE_TABLE)
#endif

I2C_STATE_TABLE:
	goto	isr_i2c_state0
	goto	isr_i2c_state1
	goto	isr_i2c_state2
	goto	isr_i2c_state3
	goto	isr_i2c_state4
	goto	isr_i2c_state5
	goto	isr_i2c_state6
	goto	isr_i2c_state7
	goto	isr_i2c_state8
	goto	isr_i2c_state9
	goto	isr_i2c_state10
	goto	isr_i2c_state11
	goto	isr_i2c_state12
I2C_MAX_STATE:

; I2C STATE 0
; We should never actually hit this. Defensive only. State 0 means idle !
;
isr_i2c_state0:
;isr_i2c_internal_error:
;	bsf	SSPCON2,PEN
;	goto	isr_i2c_error
;
; common exit code from I2C event handler and state machine
;
isr_i2c_exit:
isr_i2c_state_exit:
	movfp	UNBANK5, FSR1
isr_not_i2c:
	MOVFP 	UNBANK4, PCLATH ; Restore PCLATH value
	MOVFP 	UNBANK3, WREG ; Restore WREG value
	MOVFP 	UNBANK2, BSR ; Restore BSR value
	MOVFP 	UNBANK1, ALUSTA ; Restore ALUSTA value
;
	retfie

; ------------------------------------------------------------------------------
; internal states for above i2c code in peripheral isr routine
;
; Note I2C_STATE_0 should never occur and is located above
;      I2C_STATE_1 is handled as pat of startup routines
;
; I2C STATE 2
; on completion of I2C bus START
; If all seems OK, mark active, get bus slave address
; set the write flag and send address to bus.
; set state to 3 waiting for completion of sending of slave address
; 
isr_i2c_state2:
	btfss	SSPSTAT,S		; check start completed OK
	goto	isr_i2c_internal_error	; if not, catastrophic error
	movfp	I2C_address,WREG	; get address
	bsf	WREG,0			; set flag for bus read
	btfss	I2C_command,0		; check to see if we are writing
	bcf	WREG,0			; if so, set flag for write
	btfsc	I2C_command,1		; but check if sub address present
	bcf	WREG,0			; and if so, force to write anyway
	movwf	SSPBUF			; start sending address to bus
	movlw	3			; next state is 3
	goto	isr_i2c_state_common_exit
;
; I2C STATE 3
; on completion of sending slave address
; if slave active, either send sub address and go to state 4
; or go to simple read or write functions
; if slave inactive, send a bus STOP and go to state 0x0b ( D'11' )
;
isr_i2c_state3:
	btfsc	SSPCON2, ACKSTAT	; did we get an ACK from the slave ?
	goto	isr_i2c_state3_ackerr	; if not, go to error handler
	incf	FSR1,F			; step on to result buffer
	bsf	INDF1,2			; mark SLAVE ACTIVE
	decf	FSR1,F			; point back to state
	btfsc	I2C_command,1		; do we have a sub address to send ?
	goto	isr_i2c_do_subaddress	; if so, go send it
	btfss	I2C_command,0		; if not, simple data read or write
	goto	isr_i2c_send		; so check whether command is read or write
	goto	isr_i2c_recv		; and go to appropriate routine
isr_i2c_do_subaddress:
	movfp	I2C_subaddr,WREG	; read sub address
	movpf	WREG,SSPBUF		; and start sending it
	movlw	4			; next state is 4
	goto	isr_i2c_state_common_exit
;
isr_i2c_state3_ackerr:
	incf	FSR1,F			; point to result buffer
	bsf	INDF1,3			; mark SLAVE ACK error
	bsf	SSPCON2,PEN		; send a bus STOP
	movlw	0x0b			; next state is 0x0b
	goto	isr_i2c_state_common_exit
;
; I2C STATE 4
; sub address has been sent. If OK, check if data write.
; If so, just go on to write data to bus after subaddress
; If not, need RSEN ( Repeated start ) to re-assert address
; with flag set to READ.
;
isr_i2c_state4:
	btfsc	SSPCON2, ACKSTAT	; did we get an ACK to the sub addr
	goto	isr_i2c_ackerr		; if not, abort
	btfss	I2C_command,0		; chek for read or write
	goto	isr_i2c_send		; if write, just proceed to send
	bsf	SSPCON2,RSEN		; otherwise we need a repeated START
	movlw	5			; wait for its completion ( state 5 )
	goto	isr_i2c_state_common_exit
;
; I2C STATE 5
; We have completed the repeated start.
; If OK, resend slave address and mark as READ this time
;
isr_i2c_state5:
	btfss	SSPSTAT,S		; did start complete OK ?
	goto	isr_i2c_internal_error	; if not, error
	movfp	I2C_address,WREG	; get slave address again
	bsf	WREG,0			; mark as READ
	movwf	SSPBUF			; start sending it
	movlw	6			; next state is 6
	goto	isr_i2c_state_common_exit
;
; I2C STATE 6
; Having sent the slave address, check if we got an ACK
; If so, then start data receive operation
; and set next state to 9 for completion of receipt of byte
;
isr_i2c_state6:
	btfsc	SSPCON2, ACKSTAT	; did we get a slave ACK ?
	goto	isr_i2c_ackerr		; if not, error
;
isr_i2c_recv:
	tstfsz	I2C_count		; have we more to receive ?
	goto	isr_i2c_recv_count_not_0 ; if so, initiate data receive
	goto	isr_i2c_success		; otherwise we have got all data
isr_i2c_recv_count_not_0:
	bsf	SSPCON2,RCEN		; start a bus receive
	movlw	9			; next state 9
	goto	isr_i2c_state_common_exit
;
; I2C STATE 7
; bus operation over, bus is idle, cleanup and go to common
; exit code that cleans up and looks for something else to do
;
isr_i2c_state7:
	incf	FSR1,F			; point to result register
	bsf	INDF1,1			; mark operation as OK
	bcf	INDF1,0			; mark buffer as IDLE
	decf	FSR1,F			; point to state
	goto	isr_i2c_end		; completed command. Exit
;
; I2C STATE 8
; This is the data send handler. We stay here while
; we have more data to send.
;
isr_i2c_state8:
	btfsc	SSPCON2,ACKSTAT		; check we got a slae ACK
	goto	isr_i2c_ackerr		; and if not, abort
isr_i2c_send:
	tstfsz	I2C_count		; do we have more to send ?
	goto	isr_i2c_send_byte	; if so, skip over and send next byte
;
; bus success - we completed our data send or receive
;
isr_i2c_success:
	incf	FSR1,F			; point to result register
	bsf	INDF1,7			; mark data transfer SUCCESS
	bsf	SSPCON2,PEN		; initiate bus STOP condition
	movlw	7			; next state 7
	goto	isr_i2c_state_common_exit
;
; send a bytre of data. drop thru from above if more to send
; also enter here from other states that want to start sending
; data to bus
;
isr_i2c_send_byte:
	movfp	I2C_ram_ptr,FSR1	; set up pointer
	movfp	I2C_page, BSR		; and BSR to get data byte from RAM
	movfp	INDF1,WREG		; get it !
	movlb	BANK6			; point back to I2C registers
	movlr	BANK2			; and to ram page where we have our buffers
	movwf	SSPBUF			; start sending data byte
	incf	I2C_ram_ptr,F		; bump pointer
	decf	I2C_count,F		; decrement count
;;;	goto	isr_i2c_state_short_exit ; faulty code - remove on cleanup
	movlw	8			; next state 8 again
	goto	isr_i2c_state_common_exit
;
; I2C STATE 9
; 

isr_i2c_state9:
	movfp	SSPBUF,WREG		; get received byte of data
	movfp	I2C_ram_ptr,FSR1	; set up pointer into ram
	movfp	I2C_page, BSR		; in appropriate page
	movfp	WREG,INDF1		; and store received data
	movlb	BANK6			; reset BSR
	movlr	BANK2
	incf	I2C_ram_ptr,F		; bump pointer
	bsf	SSPCON2,ACKDT		; set flag to send NACK
	decfsz	I2C_count,F		; but check count and if not zero
	bcf	SSPCON2,ACKDT		; then change it to ACK
	bsf	SSPCON2,ACKEN		; start sending of ACK / NACK
	movlw	0x0a			; next state D'10'
	goto	isr_i2c_state_common_exit
;
; I2C STATE 0x0a ( D'10' )
; Have we received all the data we want ? After the ACK / NACK is complete
; either loop back again or jump to success handler
;
isr_i2c_state10:
	tstfsz	I2C_count
	goto	isr_i2c_recv_count_not_0
	goto	isr_i2c_success
;
; I2C STATE 0x0b ( D'11' )
; ACK error on initial send of slave address. Slave has detached
; itself from the bus, so let';s have a go at retry later.
;
isr_i2c_state11:
	btfss	I2C_command,2	; command want a retry later ?
	goto	isr_i2c_end	; if not, error exit
	movlw	-2		; mark buffer as 'retry pending'
	movwf	INDF1
	movlw	0xf0		; mask out retry counter
	andwf	I2C_command,W
	btfss	ALUSTA,Z	; and see if zero
	goto	isr_i2c_nextone	; if not, step on to next buffer
	goto	isr_i2c_end	; else clear this one and exit
;
; common code for above states when no ACK received
;
isr_i2c_ackerr:
	incf	FSR1,F		; point to result buffer
	bsf	INDF1,3		; set ACK ERROR flag
	bsf	SSPCON2,PEN	; and try to send a STOP
	movlw	0x0c		; next state 12
	goto	isr_i2c_state_common_exit


;-------------------------------------------------------------------------------		
;-------------------------------------------------------------------------------		
;
; PTT interrupt on RA0
;
; initialise ( done in init.asm )
;	set on int ->low
;	set count = 0
;
;on interrupt
;	set count to 4
;	invert int direction
;
;on 1msec ( done in 1msec interrupt handler )
;if count not zero 
;	decrement count
;	if count zero set / clear tx flag
;	if int direction does not match port status, reverse direction flag
;

IsrRA0:
;
; note following code is done in start.asm to avoid
; corrupting pclath
;
;	MOVFP 	ALUSTA, UNBANK1 ; Push ALUSTA value
;	MOVFP 	BSR, UNBANK2 	; Push BSR value
;	MOVFP 	WREG, UNBANK3 	; Push WREG value
;	MOVFP 	PCLATH, UNBANK4 ; Push PCLATH value
;
; Interrupt Service Routine (ISR) code
;
	movlr	BANK1		  ; ram page 1 for ptt_counter
	movlw	PTT_DEBOUNCE_MSEC ; set debounce counter
	movpf	WREG,ptt_counter  ; to initial value for delay 
	btg	T0STA,INTEDG

	MOVFP 	UNBANK4, PCLATH ; Restore PCLATH value
	MOVFP 	UNBANK3, WREG 	; Restore WREG value
	MOVFP 	UNBANK2, BSR 	; Restore BSR value
	MOVFP 	UNBANK1, ALUSTA ; Restore ALUSTA value
;
	retfie


;-------------------------------------------------------------------------------		
;-------------------------------------------------------------------------------		
;
IsrT0CKIF:
;	MOVFP 	ALUSTA, UNBANK1 ; Push ALUSTA value
;	MOVFP 	BSR, UNBANK2 ; Push BSR value
;	MOVFP 	WREG, UNBANK3 ; Push WREG value
;	MOVFP 	PCLATH, UNBANK4 ; Push PCLATH value
;
; Interrupt Service Routine (ISR) code
;
;	MOVFP 	UNBANK4, PCLATH ; Restore PCLATH value
;	MOVFP 	UNBANK3, WREG ; Restore WREG value
;	MOVFP 	UNBANK2, BSR ; Restore BSR value
;	MOVFP 	UNBANK1, ALUSTA ; Restore ALUSTA value
;
;	retfie

