;-------------------------------------------------------------------------------
; NAME: Keyer.asm
;
; LABEL PREFIX: "XCVR"
;
; Author:	G8KBB
;
; PURPOSE:	This is the iambic keyer.
;
;
;REVISION HISTORY:
;		03/04/01	G8KBB	Initial Revision
;
;-------------------------------------------------------------------------------		
;
; Every 10 milliseconds, execute state machine.
; change state ( or not ) depending on whether
; tx set or not or on state of keyer inputs.
;
; State machine uses following variables:
;	Keyer_state	takes values 0..7
;			use least sig 3 bits only
;	Keyer_dot	number of 10 msec intervals per dot
;	Keyer_delay	current timer count of 10 msec intervals
;	Keyer_hang	multiple of dot periods for tx/rx change
;
; and following input signals on PORTG
;	KEYER_DOT
;	KEYER_DASH
;
exec_keyer_fsm:
	movlb	BANK5			; keyer access via ports F and G
	movfp	Keyer_state,WREG	; read current state
	andlw	7			; as a precaution, limit value
	addlw	LOW keyer_state_table	; index into state jump table
	movwf	TBLPTRL
	movlw	HIGH keyer_state_table
	clrf	TBLPTRH,F
	addwfc	TBLPTRH,F
	tablrd	0,1,WREG		; dummy read loads address
	tlrd	1,PCLATH		; execute the jump
	tlrd	0,PCL
;
keyer_state_table:
	dw	keyer_state_0
	dw	keyer_state_1
	dw	keyer_state_2
	dw	keyer_state_3
	dw	keyer_state_4
	dw	keyer_state_5
	dw	keyer_state_6
	dw	keyer_state_7
;
; ------------------------------------------------------------------------
; The following are the indivdual state handlers. Every 10 msec, one
; of these is executed.

; key state 0 - idle - on receive - waiting
; if dot requested
;	request xmit mode
;	set keyer_delay = Keyer_dot * Keyer_hang
;	set state to 2
; else if dash requested
;	request xmit mode
;	set keyer_delay = Keyer_dot * Keyer_hang
;	set state to 1
; endif
;
keyer_state_0:
	btfsc	PORTG,KEYER_DOT		; if dot key not low
	goto	keyer_state_0_0		; go check for dash
	movlw	2			; dot wanted - next state 2
	goto	keyer_state_0_1		; go set up data
keyer_state_0_0:
	btfsc	PORTG,KEYER_DASH	; is dash wanted
	return				; no return
	movlw	1			; dash wanted - next state 1
keyer_state_0_1:
	movwf	Keyer_state		; set keyer state
	movlw	D'50'			; wait for at most 500 msec
	movwf	Keyer_delay		; set timeout for tx state
	bsf	RXFlags2,RXF2_KEYER_TX	; request transmit mode
	return				; and exit

;
; ------------------------------------------------------------------------
; keyer state 1 - waiting for xmit before sending dash
; if txstate == on_transmit
;	set toneon flag
;	keyer_delay = keyer_dot * 3
;	state = 3
; else
;	if keyer_delay-- == 0
;		request receive mode
;		state = 0
; 	endif
; endif

keyer_state_1:
	movlw	7			; get tx/rx fsm state - 7 means on tx
	cpfseq	tx_rx_state		; if not on transmit yet
	goto	keyer_state_1_0		; check for timeout
keyer_state_1_1:			; entry point from states 5,6
	bsf	PORTF_COPY,PWR_KEYER	; set keyer power on
	movfp	PORTF_COPY,PORTF
	movfp	Keyer_dot,WREG		; form a dash delay ( 3*dot)
	mullw	3
	movpf	PRODL,Keyer_delay	; and set as delay value
	movlw	3
	movwf	Keyer_state		; next state 3
	return
keyer_state_1_0:			; note entry point shared with keyer state 2
	decfsz	Keyer_delay,F		; not yet on tx - decrement timeout
	return				; not expired, keep waiting
keyer_state_1_2:			; entry from state 7
	bcf	RXFlags2,RXF2_KEYER_TX	; clear xmit request
	clrf	Keyer_state,F		; and return to idle state 0
	return

;
; ------------------------------------------------------------------------
; keyer state 2 - waiting for xmit before sending dot
; if txstate == on_transmit
;	set toneon flag
;	keyer_delay = keyer_dot
;	state = 4
; else
;	if keyer_delay-- == 0
;		request receive mode
;		state = 0
; 	endif
; endif

keyer_state_2:
	movlw	7			; get tx/rx fsm state - 7 means on tx
	cpfseq	tx_rx_state		; if not on xmit yet
	goto	keyer_state_1_0		; then check for timeout
keyer_state_2_1:			; entry point from states 5,6
	bsf	PORTF_COPY,PWR_KEYER	; set keyer power on
	movfp	PORTF_COPY,PORTF
	movfp	Keyer_dot,WREG		; form a dot delay
	movwf	Keyer_delay
	movlw	4			; next state 4
	movwf	Keyer_state
	return

; ------------------------------------------------------------------------
; keyer state 3 - sending dash
; if keyer_delay-- == 0
;	clear toneon flag
;	keyer_delay = keyer_dot
;	state = 5
; return
;
; if keyer is simple type, 
; stay here for as long as key down then go to state 7


keyer_state_3:
	btfss	UI_Flags, UIF_Keyer_Type ; check keyer type
	goto	keyer_simple_keydown	; if simple, jump
	decfsz	Keyer_delay,F		; still sending dash ?
	return				; if so, return
	movlw	5			; else next state 5
keyer_state_3_1:			; entry point shared state 4
	movwf	Keyer_state		; set state
	bcf	PORTF_COPY,PWR_KEYER	; turn keyer off
	movfp	PORTF_COPY,PORTF
	movfp	Keyer_dot,WREG		; set delay to one dot
	movwf	Keyer_delay
	return

keyer_simple_keydown:
	btfss	PORTG,KEYER_DASH	; is key still down ?
	return				; if so, continue
	btfss	PORTG,KEYER_DOT		; ditto other input ?
	return
	movlw	7			; no key down - go to idle
	goto	keyer_state_3_1		; reusing state 3 code

; ------------------------------------------------------------------------
; keyer state 4 - sending dot
; if keyer_delay-- == 0
;	clear toneon flag
;	keyer_delay = keyer_dot
;	state = 6
; return
;
; if keyer is simple type, 
; stay here for as long as key down then go to state 7

keyer_state_4:
	btfss	UI_Flags, UIF_Keyer_Type ; check keyer type
	goto	keyer_simple_keydown	; if simple, jump
	decfsz	Keyer_delay,F		; still sending dot ?
	return				; if so, return
	movlw	6			; else next state 6
	goto	keyer_state_3_1		; common code with state 3

; ------------------------------------------------------------------------
; keyer state 5 - delay after sending dash
; if keyer_delay-- == 0
;	if dot_requested
;		keyer_delay = keyer_dot
;		keyer_state = 4
;		set toneon flag
;	else if dash requested
;		keyer_delay = keyer_dot * 3
;		keyer_state = 3
;		set toneon flag
;	else
;		set keyer_delay = Keyer_dot * Keyer_hang
;		keyer_state = 7
;	endif
; endif

keyer_state_5:
	decfsz	Keyer_delay,F		; still in delay after dash ?
	return				; if so, return
	btfss	PORTG,KEYER_DOT		; delay over. Check for dot next
	goto	keyer_state_2_1		; and if so, go start one
	btfss	PORTG,KEYER_DASH	; otherwise check for dash
	goto	keyer_state_1_1		; and if requested, go to it
keyer_state_5_1:			; nothing indicated, set tx timeout
	movfp	Keyer_dot,WREG		; form tx timeout of hang * dot
	mulwf	Keyer_hang
	movpf	PRODL,Keyer_delay
	movlw	7			; and go into tx timeout state
	movwf	Keyer_state
	return

; ------------------------------------------------------------------------
; keyer state 6 - delay after sending dot
; if keyer_delay-- == 0
;	if dash_requested
;		keyer_delay = keyer_dot * 3
;		keyer_state = 3
;		set toneon flag
;	else if dot requested
;		keyer_delay = keyer_dot
;		keyer_state = 4
;		set toneon flag
;	else
;		set keyer_delay = Keyer_dot * Keyer_hang
;		keyer_state = 7
;	endif
; endif

keyer_state_6:
	decfsz	Keyer_delay,F		; if still in post dot delay
	return				; return
	btfss	PORTG,KEYER_DASH	; delay over - dash requested ?
	goto	keyer_state_1_1		; if so, go start one
	btfss	PORTG,KEYER_DOT		; else dot requested ?
	goto	keyer_state_2_1		; if so go start one
	goto	keyer_state_5_1		; else start tx/rx timeout

; ------------------------------------------------------------------------
; keyer state 7 - idle and on xmit
; if dash requested
;	keyer_delay = keyer_dot * 3
;	keyer_state = 3
;	set toneon flag
; else if dot requested
;	keyer_delay = keyer_dot
;	keyer_state = 4
;	set toneon flag
; else if keyer_delay-- == 0
;	request receive mode
;	state = 0
; endif

keyer_state_7:
	btfss	PORTG,KEYER_DASH	; dash requested ?
	goto	keyer_state_1_1		; if so, go start one
	btfss	PORTG,KEYER_DOT		; dot requested ?
	goto	keyer_state_2_1		; if so, go start one
	decfsz	Keyer_delay,F		; timeout happened ?
	return				; if not, keep waiting
	goto	keyer_state_1_2		; otherwise go back to receive


