;-------------------------------------------------------------------------------
; NAME: smeter.asm
;
; LABEL PREFIX: "XCVR"
;
; Author:	G8KBB
;
; PURPOSE:	This is the handler for s meter functions.
;
;
;REVISION HISTORY:
;		15/06/99	G8KBB	Initial Revision
;
;-------------------------------------------------------------------------------		
;
; Constants
;    S0 ( value to subtract from ADC reading S_Meter_ADC to get zero point )
;    Smeter_slope_A, Smeter_slope_B ( multiply / divide to get dB )
;    dBm_offset ( subtract from dB to get actual dBm value )
;    dB_per_spoint ( as it says, values 3..6 )
;    SHang ( 10's of msec s meter hang time )
;
; Variables
;    Spoint ( temp storage, use variable I, display S point )
;    s9 ( calculate on fly, as 9 * dB_per_spoint )
;    db ( calculated dB from notional zero )
;    bytes ( number of bytes of FF to display in bar graph )
;    bits ( residual number of bits after bytes of FF in bar graph )
;    SHangCount ( timer from SHang to 0 )
;
; RAM usage:
;    Own storage ( configurable ) of
;        S0
;        Smeter_slope_A
;        Smeter_slope_B
;        dBm_offset
;        dB_per_spoint
;        SHang
;    Temp storage of
;        s9 ( temp use of TBLPTRL )
;	 over ( use general variable I )
;        db ( temp use of TBLPTRH )
;        bits ( general variable K )
;        Spoint ( use general variable J )
;        bytes
;        SHangCount
;
; assumes that current ADC reading is in ram location S_Meter_ADC
;
; computes the following
;    db = ( S_Meter_ADC - S0 ) * smeter_slopeA / smeter_slopeB
;    if( db < 0 ) db = 0
;    dBm = dB - dBm_offset
;    let s9 = 9 * dB_per_spoint
;    if db > s9
;        spoint = 9
;        over = db - s9
;        bits = over * 16 / 10
;        bits += s9 * 16 / 6
;    else
;        spoint = db / dB_per_spoint
;        over = 0
;        bits = db * 16 / 6
;    endif
;    bytes = bits DIV 8
;    bits = bits MOD 8
;
; for non graphic display:
;	for dB < s9, bits = dB * 10 / 6
;	for dB > s9, bits = dBover * 9 / 6 + 9*dbperspoint*10/6
;
; note - the above is wrong and should be corrected at some time. 
; SHould be "/dbperspoint" and not "/6" in the above equations
;
; display "S"+spoint
; display "   " if over = 0,
; display "+nn" if over > 0;
; display +/- nnn dBm
; display 4 rows of graphics data thus:
;     (bytes) bytes of 0xff
;     one byte of data from lookup thus
;         bits     data
;          0        00
;          1        80
;          2        c0
;          3        e0
;          4        f0
;          5        f8
;          6        fc
;          7        fe
;
; S Meter hang time function:
; Provides an immediate attack time on meter
; and a hang time that is programmable.
; When a reading is taken, check first if hang
; time counter is zero. If it is, use the new
; reading. This is the case when either function is off
; or when hang time exceeded and update is required.
; otherwise if new >= old use the new reading and
; restart the hang timer. If the new < old, wait a
; hang time.
;
; Power meter reuses S meter code and introduces 2 extra
; configuration parameters, Pmeter_slope_A & Pmeter_slope_B.
; These operate like their S meter equivalents.
; The Power meter just reads the ADC, multiplies by A,
; divides by B and passes the reults to the bar graph function

disp_smeter:
	movlb	BANK5
	movpf	ADRESH,	WREG		; read s meter ADC count ( 0 .. 255 )
	clrf	BSR,F			; belt and braces ( may be unnecessary )
	tstfsz	SHangCount		; if hang count not zero
	goto	disp_smeter_check_ampl	; then go check new & old levels
disp_smeter_update:
	movwf	S_Meter_ADC		; we just assume conversion complete
	movfp	SHang,WREG		; initalise counter
	movwf	SHangCount		; for hang time
	goto	disp_smeter_continue	; and go process new reading
disp_smeter_check_ampl:
	cpfsgt	S_Meter_ADC		; is new >= old
	goto	disp_smeter_update	; if so go update old with new
	decf	SHangCount,F		; else decrement count
	return				; and make no display change
disp_smeter_continue:
	movlw	7			; check if on xmit ( state 7 )
	cpfseq	tx_rx_state		; if so we do power meter
	goto	disp_smeter_continue2
	goto	disp_power_meter
disp_smeter_continue2:
	tstfsz	Smeter_slope_A		; s meter is off if slope_A is zero
	goto	disp_smeter_active
	return
disp_smeter_active:
	movfp	S0,WREG			; read S0 offset
	btfsc	RXFlags2,RXF2_CWFilter	; check if offset is for CW filter or not
	movfp	S0CW,WREG		; and if so use other S0 level
	subwf	S_Meter_ADC,W		; offset it to zero
	btfss	ALUSTA,C		; if no carry, result negative
	clrf	WREG,F			; else force to zero
	mulwf	Smeter_slope_A		; multiply by slope_A
	movpf	PRODH,AARGB0		; and divide by slope_B
	movpf	PRODL,AARGB1
	movfp	Smeter_slope_B,WREG
	movpf	WREG,BARGB0
	btfss	ALUSTA,Z		; ( but only if divisor non zero )
	call	FXD1507U
	;
	; AARGB0:1 contain dB reading
	; but we assume ( and require ) 8 bit result only in AARGB1
	;
	; now correct for band and attenuator
	;
	movfp	AARGB1,WREG
	addwf	Atten_cal,W		; add attenuator correction
	addwf	SMeter_cal,W		; add band correction
	btfsc	WREG,7			; result negative ?
	movlw	0			; if so, set to zero
	movwf	AARGB1
;
	movfp	AARGB1,TBLPTRH		; save dB for later
	movfp	dB_per_spoint,WREG	; form s9 = dB_per_spoint * 9
	mullw	9
	movpf	PRODL,TBLPTRL		; and store temporarily
	movfp	PRODL,WREG
	cpfsgt	AARGB1
	goto	disp_smeter_0		; jump if db <= s9
	movlw	9
	movwf	J			; spoint = 9
	movfp	TBLPTRL,WREG
	subwf	AARGB1,W		; over = dB - s9
	movwf	I			; and store in I
#ifdef NON_GRAPHIC_SMETER
	mullw	9			; non graphic bits = over * 9...
#else
	mullw	0x10			; else bits = over * 16.....
#endif
	movpf	PRODH,AARGB0		; ... and divide by 10 or 6
	movpf	PRODL,AARGB1		; for graphics / non graphics
#ifdef NON_GRAPHIC_SMETER
	movlw	6
#else
	movlw	0x0a
#endif
	movwf	BARGB0
	call	FXD1507U		; result is in AARGB1
	movfp	AARGB1,WREG
	movwf	K			; store bits in K
	movfp	TBLPTRL,WREG		; get s9 in dB
; common code to disp_smeter_0
;	mullw	0x10			; s9 * 16
;	movpf	PRODH,AARGB0		; ... and divide by 6
;	movpf	PRODL,AARGB1
;	movlw	6
;	movpf	BARGB0
;	call	FXD1507U		; result is in AARGB1
;	movfp	AARGB1,WREG
;	addwf	K,F			; add into bits
	goto	disp_smeter_1
disp_smeter_0:
	movfp	dB_per_spoint,WREG	; we still have dB in AARGB0:1
	movwf	BARGB0			; divide by dB_per_spoint
	call	FXD1507U		; to get S points
	movfp	AARGB1,WREG
	movwf	J			; store s point
	clrf	I,F			; over = 0
	clrf	K,F			; bits = 0 ( allow for addwf below )
	movfp	TBLPTRH,WREG
disp_smeter_1:
#ifdef NON_GRAPHIC_SMETER
	mullw	0x0a			; form dB * 10 ( non graphic )
#else
	mullw	0x10			; form dB * 16
#endif
	movpf	PRODH,AARGB0		; ... and divide by 6
	movpf	PRODL,AARGB1
	movlw	6
	movwf	BARGB0
	call	FXD1507U		; result is in AARGB1
	movfp	AARGB1,WREG
	addwf	K,F			; add into bits
	btfss	ALUSTA,C		; check for overflow
	goto	disp_smeter_no_ovf
	movlw	0xff			; and if so, force to 255
	movwf	K
disp_smeter_no_ovf:
;
; we now have following
;	S point reading in J
;	dB in TBLPTRH
;	bits in bargraph in K
;	dB 'over' S9 in I
;
	movfp	J,WREG			; get S points
	addlw	'0'			; change to ASCII
	movwf	DISP_BUF		; store as data to display
	tstfsz	I			; check 'over'
	goto	disp_smeter_over	; display +nn
	movlw	' '			; display "   "
	movwf	DISP_BUF+1
	movwf	DISP_BUF+2
	movwf	DISP_BUF+3
	goto	disp_smeter_2
disp_smeter_over:
	movfp	I,WREG			; get dB over
	movwf	AARGB1
	clrf	AARGB0,F		; AARGB := over
	movlw	0x0A
	movwf	BARGB0			; BARGB := 10
	call	FXD1507U		; convert for display
	movlw	'+'			; '+'
	movwf	DISP_BUF+1
	movfp	AARGB1,WREG		; 'n' ( ms digit )
	addlw	'0'
	movwf	DISP_BUF+2
	movfp	REMB0,WREG		; 'n' ( ls digit )
	addlw	'0'
	movwf	DISP_BUF+3
disp_smeter_2:				; print space next
	movlw	' '
	movwf	DISP_BUF+4
	movpf	TBLPTRH,WREG
	cpfslt	dBm_offset
	goto	disp_smeter_3
	movlw	'+'			; now + or - for dBm
	movwf	DISP_BUF+5
	movfp	dBm_offset,WREG
	subwf	TBLPTRH,W
	goto	disp_smeter_4
disp_smeter_3:
	movlw	'-'
	movwf	DISP_BUF+5
	movpf	TBLPTRH,WREG
	subwf	dBm_offset,W
disp_smeter_4:
	movwf	AARGB1
	clrf	AARGB0,F		; AARGB := over
	movlw	0x0A
	movwf	BARGB0			; BARGB := 10
	call	FXD1507U		; convert for display
	movfp	REMB0,WREG
	addlw	'0'
	movwf	DISP_BUF+8
	call	FXD1507U		; convert for display
	movfp	REMB0,WREG
	addlw	'0'
	movwf	DISP_BUF+7
	movfp	AARGB1,WREG
	addlw	'0'
	movwf	DISP_BUF+6
#ifdef NON_GRAPHIC_SMETER
	clrf	DISP_BUF+9,F 
#else
	movlw	'd'
	movwf	DISP_BUF+9
	movlw	'B'
	movwf	DISP_BUF+0x0A
	movlw	'm'
	movwf	DISP_BUF+0x0B
	movlw	0x20
	movwf	DISP_BUF+0x0C
	clrf	DISP_BUF+0x0D,F
#endif
	movlw	s_smeter3-string_index
	call	p_string_xy

;
; now display bar graph
; NOTE - it is assumed that K is preserved in p_string_xy !
; also this code is common to S and power meters
;--------------------------------------------------
; for simple bar graph ( non graphic ):
; if bits > max bits
;	bits = maxbits
; set cursor to start position
; while( bits > 4 )
;	sendbyte( 4 ) // note cgram - see table in lcd.asm
;	bits -= 5
; if( bits > 0 )
;	sendbyte( bytes )
;-------------------------------------------------- 
; for graphic s meter:
; if bits > max bits
;	bits = maxbits
; bytes = bits / 8
; bits = bits mod 8
; for i = 0 to 3
;	set cursor to x,y+i
;	for j = 0 to bytes
;		sendbyte( ff )
;	sendbyte( bitlookup[bits] )
;	while j < maxbytes
;		sendbyte( 0 )
;		j = j + 1
;
; variables:
;	I,J used for x,y scren coordinates
;	K still holds bits
;	TEMP0 used for bitlookup[bits]
;	TEMP1 used for loop i
;	TEMP2 used for loop j ( first loop )
;	TEMP3 used for loop j ( second loop )
;
meter_bargraph:
	movlw	SMETERMAXBITS		; get maxbits
	cpfslt	K			; is bits >= maxbits ?
	movwf	K			; if so, set bits = maxbits
#ifdef NON_GRAPHIC_SMETER
	LOAD_XCOORD ( SMETERXLOC, SMETERYLOC )
	movwf	I
	LOAD_YCOORD ( SMETERXLOC, SMETERYLOC )
	movwf	J
	call	LCDGOTOXY		; move cursor to start of s meter

	; in this loop, we use the fact that we have a
	; 5 column per character display ( i.e. 7x5 )
	; and output a full bar for each 5 bits 
	; we do this by subtracting 5 each time until
	; we have < 5 bits left, then use the fact
	; that the char gen graphics characters will
	; correctly display is we just use the remaining
	; count as the character to display
	; note also that we need to clear the rest of the
	; row.

	clrf	I,F			; use this as byte count to clear rest of line
meter_bargraph_loop:
	movlw	4			; char with all 5 pixels set
	cpfsgt	K			; > 4 ? then keep going
	goto	meter_bargraph_endloop
	call	LCDDATA			; display full 5 pixels ( char code 4 )
	movlw	5			; subtract 5 from bit count
	subwf	K,F
	incf	I,F			; count how many we've done
	goto	meter_bargraph_loop	; and loop
meter_bargraph_endloop:
	movfp	K,WREG			; end loop - now just display remaining bits 
	tstfsz	K			; if any
	call	LCDDATA			; again, notr correlation between K and character code
	tstfsz	K			; and also bump count if we sent another char
meter_bargraph_clear_loop:
	incf	I,F
	movlw	SMETERMAXBYTES
	cpfslt	I			; reached the end ?
	return				; if so, exit
	movlw	0x20
	call	LCDDATA
	goto	meter_bargraph_clear_loop
#else

	; the corresponding program for a graphics display
	; is more complex

	rrcf	K,W
	rrcf	WREG,F
	rrcf	WREG,F
	andlw	0x1f			; get bits / 8 in WREG
	movwf	TEMPB2			; store bytes as first loop counter
	sublw	SMETERMAXBYTES		; form second loop counter
	movwf	TEMPB3
	movlw	4
	movwf	TEMPB1			; form outer loop counter j
	movlw	7
	andwf	K,F			; form bits = bits mod 8
	clrf	WREG,F			; now do conversion to bit mask
	btfsc	ALUSTA,Z		; bits = 0 means result 0
	goto 	smeter_bitlookup_end	; so skip over the loop
smeter_bit_lookup_loop:
	bsf	ALUSTA,C		; set so rotate sets msbit each time
	rrcf	WREG,F			; and create result so that 1 gives
	decfsz	K,F			; result 0x80, 2 gives 0xc0, 3 0xe0,
	goto	smeter_bit_lookup_loop	; 4 0xf0, 5 0xf8, 6 0xfc and 7 0xfe
smeter_bitlookup_end:
	movwf	TEMPB0			; store result for use in loops below

smeter_bar_loop:
	movfp	TEMPB1,WREG
	addlw	SMETERYLOC		; Y location on screen
	mullw	LCD_COLS
	movlw	SMETERXLOC		; x location on screen
	addwf	PRODL,F
	movlw	0
	addwfc	PRODH,F		
	movpf	PRODH,J			; set 'y' for cursor move
	movpf	PRODL,I			; set 'x' for cursor move
	call	LCDGOTOXYGRAPH
	movfp	TEMPB2, PRODH		; set loop counter for j=0 to bytes
	tstfsz	PRODH			; check for 0
	goto	smeter_loop_1		; need to send some bytes of ff
	goto	smeter_loop_2		; no ff to send as count is zero
smeter_loop_1:
	movlw	0xff			; send byte of 0xff
	call	LCDDATAFAST		; send byte
	decfsz	PRODH,F			; and loop round j times
	goto 	smeter_loop_1
smeter_loop_2:
	movfp	TEMPB0,WREG		; send bitlookup[bits]
	call	LCDDATAFAST		; send byte
	movfp	TEMPB3, PRODH		; set loop counter for j=0 to bytes
	tstfsz	PRODH			; check for 0
	goto	smeter_loop_3		; need to send some bytes of ff
	goto	smeter_loop_4		; no ff to send as count is zero
smeter_loop_3:
	movlw	0x00			; send byte of 0x00
	call	LCDDATAFAST		; send byte
	decfsz	PRODH,F			; and loop j2 times
	goto 	smeter_loop_3
smeter_loop_4:
	decfsz	TEMPB1,F		; send 4 rows of bar graph
	goto	smeter_bar_loop
	return
#endif

;-------------------------------------------------------------------------------		

disp_power_meter:
	tstfsz	Pmeter_slope_A		; power meter is off if slope_A is zero
	goto	disp_pmeter_active
	return
disp_pmeter_active:
	movfp	S_Meter_ADC,WREG	; get power meter ADC reading
	mulwf	Pmeter_slope_A		; multiply by slope_A
	movpf	PRODH,AARGB0		; and divide by slope_B
	movpf	PRODL,AARGB1
	movfp	Pmeter_slope_B,WREG
	movpf	WREG,BARGB0
	btfss	ALUSTA,Z		; ( but only if divisor non zero )
	call	FXD1507U
	movfp	AARGB1,WREG
	movwf	K
	goto	meter_bargraph
	; return implicit in goto


